module Errors
    class DLLNotFoundError < StandardError
        def message
            "Could not find DLL. Is Voicemeeter installed correctly?"
        end
    end

    class EXENotFoundError < StandardError
        def message
            "Could not find EXE. Is Voicemeeter installed correctly?"
        end
    end

    class VBTypeError < StandardError
        def on_launch
            "Login was requested but no Voicemeeter type was provided,
            nor was Voicemeeter running. To fix do either of the following:
            a) Provide a Voicemeeter type when you call Remote.new
            b) Ensure that Voicemeeter is running before your program starts"
        end

        def message
            "Unknown Voicemeeter type"
        end
    end

    class LoginError < StandardError
        def message
            "Failed to login, success return value:"
        end
    end

    class LogoutError < StandardError
        def message
            "There was an error logging out"
        end
    end

    class APIError < StandardError
        def message
            "Callback Function Error, return value:"
        end
    end

    class BoundsError < StandardError
        def message
            "Value out of bounds"
        end
    end

    class VersionError < StandardError
        def message
            "Wrong Voicemeeter version"
        end
    end

    class CommandError < StandardError
        def message
            "Command not supported"
        end
    end

    class ValueTypeError < StandardError
        def message
            "Incorrect value type"
        end
    end

    class ParamValueError < StandardError
        def message
            "Parameter value out of range"
        end
    end

    class WriteError < StandardError
        def message
            "Parameter is read only"
        end
    end
end
